package com.laguerre.mytest;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import com.laguerre.kan.Camera;
import com.laguerre.kan.Color;
import com.laguerre.kan.Light;
import com.laguerre.kan.Lights;
import com.laguerre.kan.Material;
import com.laguerre.kan.MeshRenderer;
import com.laguerre.kan.ParticleSystem;
import com.laguerre.kan.ParticleSystemRenderer;
import com.laguerre.kan.PassShader;
import com.laguerre.kan.Quaternion;
import com.laguerre.kan.RenderTexture;
import com.laguerre.kan.RenderingSettings;
import com.laguerre.kan.SceneConductor;
import com.laguerre.kan.SceneObject;
import com.laguerre.kan.Shader;
import com.laguerre.kan.SurfaceInfo;
import com.laguerre.kan.Texture;
import com.laguerre.kan.Texture2D;
import com.laguerre.kan.Utils;
import com.laguerre.kan.Vector3;
import com.laguerre.kan.ParticleSystem.Burst;

import android.content.Context;
import android.graphics.Bitmap;
import android.opengl.GLSurfaceView;

public class MyGL20Renderer implements GLSurfaceView.Renderer {
	private Context context;

	public MyGL20Renderer(Context context) {
		this.context = context;
		// To be accessible later, somewhere else...
		Utils.setContext(context);
	}

	@Override
    public void onSurfaceCreated(GL10 unused, EGLConfig config) {
		SurfaceInfo.resetInstance();
		RenderingSettings.resetInstance();
		Lights.resetInstance();
		SceneConductor.resetInstance();

		buildWorld();
	}

	private SceneObject camSob0;

	private SceneObject cubeCam0;
	private SceneObject camSob1;

	private void buildWorld() {
		RenderingSettings.getInstance().setAmbientLight(new Color(0.1f, 0.1f, 0.1f, 1f));

		// Layer 1  (i.e. a camera will inject what will be see in this layer to the layer).
		//
//		final int layer1 = 1 << 1;
//		final SceneObject myCamera1 = SceneConductor.getInstance().addSceneObject("Camera 1");
//		myCamera1.getSharedTransform().translateGlobally(-0.6f, 0f, 1f);
//		myCamera1.addComponent(Camera.class);
//		myCamera1.addComponent(PerlinImageEffect.class);
//		myCamera1.getSharedCamera().setCullingMask(layer1);
//		this.camSob1 = myCamera1;
//
//		addQuad(new Vector3(-0.5f, 0.2f, -4f)).setLayer(layer1);
//
//		final SceneObject lightSobQuad = SceneConductor.getInstance().addSceneObject("Light quad");
//		lightSobQuad.setLayer(layer1);
//		lightSobQuad.getSharedTransform().translateGlobally(0f, 0.2f, -3f);
//		final Light lightQuad = lightSobQuad.addComponent(Light.class);
//		lightQuad.setLightType(Light.LightType.POINT);
//		lightQuad.setConstantAttenuation(1.0f);
//		lightQuad.setLinearAttenuation(0.15f);
//		lightQuad.setQuadraticAttenuation(0.8f);
//		// Very red to see the quad once projected on the cube of the camera0.
//		lightQuad.setDiffuseColor(0.8f, 0.0f, 0.0f, 0.8f);
//		lightQuad.setSpecularColor(0.9f, 0.0f, 0.0f, 0.9f);


		// Now, layer 0 (it will be rendered on screen)
		//
		final SceneObject myCamera0 = SceneConductor.getInstance().addSceneObject("Camera 0");
		//myCamera0.getSharedTransform().translateGlobally(0.3f, 0f, 5f);
		myCamera0.getSharedTransform().translateGlobally(0.3f, 0f, 6f);
		//myCamera0.getSharedTransform().rotateGlobally(new Quaternion(90f, Vector3.RIGHT));
		myCamera0.addComponent(Camera.class);
		// Apply a post-effect.
		//myCamera0.addComponent(PerlinImageEffect.class);
		//myCamera0.addComponent(GaussianBlurImageEffect.class);
		//myCamera0.addComponent(LaplaceEdgeImageEffect.class);
		myCamera0.getSharedCamera().setNearAndFarClipPlanes(0.3f, 8f);
		// Makes this camera, the camera that renders to screen.
		myCamera0.getSharedCamera().setAsRenderingOnScreen();
		this.camSob0 = myCamera0;

		this.cubeCam0 = addCube(new Vector3(0.5f, 0.2f, -1.5f));

		final SceneObject lightSob0 = SceneConductor.getInstance().addSceneObject("Light 0");
		lightSob0.getSharedTransform().translateGlobally(0.5f, 0.2f, 1f);
		final Light light0 = lightSob0.addComponent(Light.class);
		light0.setLightType(Light.LightType.SPOT);
		light0.setDiffuseColor(0.5f, 0.5f, 0.4f, 0.7f);
		light0.setSpecularColor(0.8f, 0.8f, 0.75f, 0.8f);
		light0.setRange(-1f);
		light0.setConstantAttenuation(0f);
		light0.setLinearAttenuation(0f);
		light0.setQuadraticAttenuation(0.1f);
		light0.setInnerCutOffAngle(2f);
		light0.setOuterCutOffAngle(10f);

		final SceneObject lightSob1 = SceneConductor.getInstance().addSceneObject("Light 1");
		lightSob1.getSharedTransform().translateGlobally(0f, 0.4f, -0.5f);
		Quaternion q = new Quaternion(-90f, Vector3.UP);
		q.mulSelf(new Quaternion(-30f, Vector3.RIGHT));
		lightSob1.getSharedTransform().rotateGlobally(q);
		final Light light1 = lightSob1.addComponent(Light.class);
		light1.setLightType(Light.LightType.DIRECTIONAL);
		light1.setDiffuseColor(0.8f, 0.8f, 0.5f, 1f);
		light1.setSpecularColor(1.0f, 1.0f, 0.7f, 1f);

		final SceneObject particleSob = SceneConductor.getInstance().addSceneObject("Particles");
		particleSob.getSharedTransform().setParent(this.cubeCam0.getSharedTransform());
		particleSob.getSharedTransform().rotateLocally(new Quaternion(90f, Vector3.RIGHT));
		particleSob.getSharedTransform().translateLocally(-0.15f, -0.4f, -0.5f);
		final ParticleSystemRenderer particleRenderer = particleSob.addComponent(ParticleSystemRenderer.class);
		final Material particleMat = new Material(0);
		particleRenderer.setMaterial(particleMat, 0);
		final Shader shaderMatParticle = new Shader();
		particleMat.setShader(shaderMatParticle);
		SceneConductor.getInstance().addSceneObjectToRenderQueue(
				particleSob, Shader.RenderQueueType.TRANSPARENT.index());
		final PassShader passShader0MatParticle = shaderMatParticle.addPassShader(0);
		passShader0MatParticle.setDepthWriting(false);
		passShader0MatParticle.setBlending(true);
		passShader0MatParticle.setBlendFunc(PassShader.BlendSrc.ALPHA, PassShader.BlendDst.ALPHA);
		passShader0MatParticle.createProgram();
		passShader0MatParticle.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.simple_particle_v_glsl));
		passShader0MatParticle.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.simple_particle_f_glsl));
		passShader0MatParticle.linkProgram();
		final Texture2D mainTexParticle = new Texture2D();
		mainTexParticle.setBitmap(this.context, R.drawable.round_particle, Bitmap.Config.ARGB_8888);
		mainTexParticle.setMinifyingFilter(Texture.MinifyingFilter.LINEAR);
		mainTexParticle.setMagnificationFilter(Texture.MagnificationFilter.LINEAR);
		mainTexParticle.create();
		particleMat.setShaderTexture("mainTex", mainTexParticle);
		final ParticleSystem particleSystem = particleSob.addComponent(ParticleSystem.class);

//		final ParticleSystem.SphereShape sphere = new ParticleSystem.SphereShape();
//		sphere.setRadius(0.5f);
//		sphere.setEmitFrom(ParticleSystem.SphereShape.EmitFrom.CENTER);
//		particleSystem.setEmitterShape(sphere);

		final ParticleSystem.ConeShape cone = new ParticleSystem.ConeShape();
		cone.setUnitConeAngle(30f);
		cone.setEmitFrom(ParticleSystem.ConeShape.EmitFrom.ORIGIN);
		particleSystem.setEmitterShape(cone);

		final ParticleSystem.ApplyVelocityModule applyVelocityModule = new ParticleSystem.ApplyVelocityModule();
		applyVelocityModule.setApplyGravity(false);
		particleSystem.addParticleModule(applyVelocityModule);
		final ParticleSystem.UpdateColorOverTimeModule colorOverTime = new ParticleSystem.UpdateColorOverTimeModule();
		particleSystem.addParticleModule(colorOverTime);
		colorOverTime.setInitialColor(1f, 0f, 0f, 1f);
		colorOverTime.setFinalColor(0.2f, 0.6f, 0.9f, 0f);

		particleSystem.setStartLifeTime(4f);
		//particleSystem.setStartColor(Color.GREEN);
		//particleSystem.emit(2);
		particleSystem.setRateEmission(20);
		particleSystem.setStartSize(0.15f);
		particleSystem.setStartRotation(-30f);
		//particleSystem.setStartSpeed(1.3f);
		particleSystem.setStartSpeed(0.4f);

		final Burst burst0 = new Burst();
		burst0.setAmount(50);
		burst0.setTimeAtBurst(5f);
		particleSystem.addBurst(burst0);

		particleSystem.play();
	}

	private SceneObject addTriangle(Vector3 translation) {
		final SceneObject sob = SceneConductor.getInstance().addSceneObject("Triangle");
		sob.getSharedTransform().translateGlobally(translation);

		final SimpleTriangle mesh = sob.addComponent(SimpleTriangle.class, "Simple triangle");
		MeshRenderer sobRenderer = sob.addComponent(MeshRenderer.class, "Mesh renderer");

		final Material sobMat = new Material(0);
		final Shader shader = new Shader();
		sobMat.setShader(shader);
		SceneConductor.getInstance().addSceneObjectToRenderQueue(
				sob, Shader.RenderQueueType.GEOMETRY.index());
		final PassShader passShader0 = shader.addPassShader(0);
		passShader0.setDepthTest(PassShader.DepthTest.LEQUAL);
		passShader0.createProgram();
		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_v_glsl));
		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_f_glsl));
		passShader0.linkProgram();

		final Texture2D mainTex = new Texture2D();
		mainTex.setBitmap(this.context, R.drawable.carte_mcdo, Bitmap.Config.ARGB_8888);
		mainTex.create();
		sobMat.setShaderTexture("mainTex", mainTex);
		sobMat.setShaderVector("specularColor", Color.WHITE);
		sobMat.setShaderValue("shininess", 5f);

		sobRenderer.setMaterial(sobMat);

		mesh.init();

		return sob;
	}

	private SceneObject addQuad(Vector3 translation) {
		final SceneObject sob = SceneConductor.getInstance().addSceneObject("Quad");
		sob.getSharedTransform().translateGlobally(translation);

		final SimpleQuad mesh = sob.addComponent(SimpleQuad.class, "Simple quad");
		MeshRenderer sobRenderer = sob.addComponent(MeshRenderer.class, "Mesh renderer");

		final Material sobMat = new Material(0);
		final Shader shader = new Shader();
		sobMat.setShader(shader);
		SceneConductor.getInstance().addSceneObjectToRenderQueue(
				sob, Shader.RenderQueueType.GEOMETRY.index());
		final PassShader passShader0 = shader.addPassShader(0);
		passShader0.setDepthTest(PassShader.DepthTest.LEQUAL);
		passShader0.createProgram();
		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_v_glsl));
		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_f_glsl));
		passShader0.linkProgram();

		final Texture2D mainTex = new Texture2D();
		mainTex.setBitmap(this.context, R.drawable.carte_mcdo, Bitmap.Config.ARGB_8888);
		mainTex.create();
		sobMat.setShaderTexture("mainTex", mainTex);
		sobMat.setShaderVector("specularColor", Color.WHITE);
		sobMat.setShaderValue("shininess", 5f);

		sobRenderer.setMaterial(sobMat);

		mesh.init();

		return sob;
	}

	private SceneObject addCube(Vector3 translation) {
		final SceneObject sobParent = SceneConductor.getInstance().addSceneObject("Cube parent");
		sobParent.getSharedTransform().translateGlobally(translation);

		final SceneObject sobCube = SceneConductor.getInstance().addSceneObject("Cube");
		sobCube.getSharedTransform().setParent(sobParent.getSharedTransform());

		final Quaternion q = new Quaternion(30f, Vector3.RIGHT);
		q.mulSelf(new Quaternion(15f, Vector3.UP));
		sobCube.getSharedTransform().rotateLocally(q);

		final Cube mesh = sobCube.addComponent(Cube.class, "Cube");
		MeshRenderer sobRenderer = sobCube.addComponent(MeshRenderer.class, "Mesh renderer");

		final Material sobMat = new Material(0);
		final Shader shader = new Shader();
		sobMat.setShader(shader);
		SceneConductor.getInstance().addSceneObjectToRenderQueue(
				sobCube, Shader.RenderQueueType.GEOMETRY.index());
		final PassShader passShader0 = shader.addPassShader(0);
		passShader0.setDepthTest(PassShader.DepthTest.LEQUAL);
		passShader0.createProgram();

//		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
//				this.context, R.raw.one_tex_v_glsl));
//		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
//				this.context, R.raw.one_tex_f_glsl));

		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.normal_mapping_v_glsl));
		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.normal_mapping_f_glsl));
		passShader0.linkProgram();

		final Texture2D mainTex = new Texture2D();
		mainTex.setBitmap(this.context, R.drawable.carte_mcdo, Bitmap.Config.ARGB_8888);
		mainTex.create();
		final Texture2D normalTex = new Texture2D();
		normalTex.setBitmap(this.context, R.drawable.carte_mcdo_normals, Bitmap.Config.ARGB_8888);
		normalTex.create();
		sobMat.setShaderTexture("mainTex", mainTex);
		sobMat.setShaderTexture("normalTex", normalTex);
		sobMat.setShaderVector("specularColor", Color.WHITE);
		sobMat.setShaderValue("shininess", 5f);

		sobRenderer.setMaterial(sobMat);

		mesh.init();

		return sobCube;
	}

	private boolean surfaceDimKnown = false;

	@Override
    public void onDrawFrame(GL10 unused) {
		if (this.surfaceDimKnown) {
	        SceneConductor.getInstance().drawFrame();
		}
    }

	@Override
    public void onSurfaceChanged(GL10 unused, int width, int height) {
    	this.surfaceDimKnown = true;

		final RenderTexture camRenderTex0 = new RenderTexture();
		camRenderTex0.init(width, height);
		camRenderTex0.create();
		this.camSob0.getSharedCamera().setRenderTexture(camRenderTex0);

//		final RenderTexture camRenderTex1 = new RenderTexture();
//		camRenderTex1.init(width, height);
//		camRenderTex1.create();
//		this.camSob1.getSharedCamera().setRenderTexture(camRenderTex1);
//
//		final Material mat = this.cubeCam0.getSharedRenderer().getSharedMaterial(0);
//		mat.setShaderTexture("mainTex", camRenderTex1);

		SurfaceInfo.getInstance().setSurfaceSize(width, height);

		SceneConductor.getInstance().onRenderSurfaceChanged(width, height);
    }
}
